@@ -124,13 +124,13 @@ sub run {
 
     # If the profile parameter has been set in the API, then load a profile
     if ( $params->{profile} ) {
-    if ( $params->{profile} eq 'test_profile_1' and Zonemaster::Backend::Config->CustomProfilesPath()) {
-        # The config has defined an alternative profile and "test_profile_1" has been set.
-        Zonemaster::Engine->config->load_policy_file( Zonemaster::Backend::Config->CustomProfilesPath() . '/iana-profile.json' );
-    }
-    else { # The profile parameter has been set to something else or alternative profile is not defined
-        Zonemaster::Engine->config->load_policy_file( 'iana-profile.json' );
-    }
+        if ( $params->{profile} eq 'test_profile_1' and Zonemaster::Backend::Config->CustomProfilesPath()) {
+            # The config has defined an alternative profile and "test_profile_1" has been set.
+            Zonemaster::Engine->config->load_policy_file( Zonemaster::Backend::Config->CustomProfilesPath() . '/policy.json' );
+        }
+        else { # The profile parameter has been set to something else or alternative profile is not defined
+            Zonemaster::Engine->config->load_policy_file( 'policy.json' );
+        }
     # It will be silently ignored if the file does not exist.
     }
 
